@@ -2,7 +2,8 @@ from __future__ import annotations
 
 import os
 import sys
-from typing import TYPE_CHECKING, cast
+from functools import cached_property
+from typing import TYPE_CHECKING, Any, cast
 
 from hatch.cli.terminal import Terminal
 from hatch.config.user import ConfigFile, RootConfig
@@ -13,11 +14,13 @@ from hatch.utils.platform import Platform
 if TYPE_CHECKING:
     from packaging.requirements import Requirement
 
+    from hatch.env.plugin.interface import EnvironmentInterface
+
 
 class Application(Terminal):
     def __init__(self, exit_func, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self.platform = Platform(self.display_raw)
+        self.platform = Platform(self.output)
         self.__exit_func = exit_func
 
         self.config_file = ConfigFile()
@@ -39,7 +42,7 @@ class Application(Terminal):
     def config(self) -> RootConfig:
         return self.config_file.model
 
-    def get_environment(self, env_name=None):
+    def get_environment(self, env_name: str | None = None) -> EnvironmentInterface:
         if env_name is None:
             env_name = self.env
 
@@ -67,53 +70,92 @@ class Application(Terminal):
             self.get_safe_application(),
         )
 
+    def prepare_internal_environment(self, env_name: str, config: dict[str, Any] | None = None) -> EnvironmentInterface:
+        from hatch.env.internal import get_internal_environment_class
+
+        environment_class = get_internal_environment_class(env_name)
+        storage_dir = self.data_dir / 'env' / '.internal' / env_name
+        environment = environment_class(
+            self.project.location,
+            self.project.metadata,
+            env_name,
+            config or {},
+            {},
+            storage_dir,
+            storage_dir,
+            self.platform,
+            self.verbosity,
+            self.get_safe_application(),
+        )
+        try:
+            environment.check_compatibility()
+        except Exception as e:  # noqa: BLE001
+            self.abort(f'Internal environment `{env_name}` is incompatible: {e}')
+
+        self.prepare_environment(environment)
+        return environment
+
     # Ensure that this method is clearly written since it is
     # used for documenting the life cycle of environments.
-    def prepare_environment(self, environment):
+    def prepare_environment(self, environment: EnvironmentInterface):
         if not environment.exists():
-            with self.status_waiting(f'Creating environment: {environment.name}'):
+            self.env_metadata.reset(environment)
+
+            with self.status(f'Creating environment: {environment.name}'):
                 environment.create()
 
             if not environment.skip_install:
                 if environment.pre_install_commands:
-                    with self.status_waiting('Running pre-installation commands'):
+                    with self.status('Running pre-installation commands'):
                         self.run_shell_commands(environment, environment.pre_install_commands, source='pre-install')
 
                 if environment.dev_mode:
-                    with self.status_waiting('Installing project in development mode'):
+                    with self.status('Installing project in development mode'):
                         environment.install_project_dev_mode()
                 else:
-                    with self.status_waiting('Installing project'):
+                    with self.status('Installing project'):
                         environment.install_project()
 
                 if environment.post_install_commands:
-                    with self.status_waiting('Running post-installation commands'):
+                    with self.status('Running post-installation commands'):
                         self.run_shell_commands(environment, environment.post_install_commands, source='post-install')
 
-        with self.status_waiting('Checking dependencies'):
-            dependencies_in_sync = environment.dependencies_in_sync()
+        dep_hash = environment.dependency_hash()
+        current_dep_hash = self.env_metadata.dependency_hash(environment)
+        if dep_hash != current_dep_hash:
+            with self.status('Checking dependencies'):
+                dependencies_in_sync = environment.dependencies_in_sync()
 
-        if not dependencies_in_sync:
-            with self.status_waiting('Syncing dependencies'):
-                environment.sync_dependencies()
+            if not dependencies_in_sync:
+                with self.status('Syncing dependencies'):
+                    environment.sync_dependencies()
+
+            self.env_metadata.update_dependency_hash(environment, dep_hash)
 
     def run_shell_commands(
-        self, environment, commands: list[str], source='cmd', *, force_continue=False, show_code_on_error=True
+        self,
+        environment: EnvironmentInterface,
+        commands: list[str],
+        source='cmd',
+        *,
+        force_continue=False,
+        show_code_on_error=True,
     ):
         with environment.command_context():
             try:
                 resolved_commands = list(environment.resolve_commands(commands))
-            except Exception as e:
+            except Exception as e:  # noqa: BLE001
                 self.abort(str(e))
 
             first_error_code = None
             should_display_command = self.verbose or len(resolved_commands) > 1
-            for i, command in enumerate(resolved_commands, 1):
+            for i, raw_command in enumerate(resolved_commands, 1):
                 if should_display_command:
-                    self.display(f'{source} [{i}] | {command}')
+                    self.display(f'{source} [{i}] | {raw_command}')
 
+                command = raw_command
                 continue_on_error = force_continue
-                if command.startswith('- '):
+                if raw_command.startswith('- '):
                     continue_on_error = True
                     command = command[2:]
 
@@ -122,7 +164,8 @@ class Application(Terminal):
                     first_error_code = first_error_code or process.returncode
                     if continue_on_error:
                         continue
-                    elif show_code_on_error:
+
+                    if show_code_on_error:
                         self.abort(f'Failed with exit code: {process.returncode}', code=process.returncode)
                     else:
                         self.abort(code=process.returncode)
@@ -130,44 +173,6 @@ class Application(Terminal):
             if first_error_code and force_continue:
                 self.abort(code=first_error_code)
 
-    def attach_builder(self, process):
-        import pickle
-
-        with process:
-            for line in self.platform.stream_process_output(process):
-                indicator, _, procedure = line.partition(':')
-                if indicator != '__HATCH__':  # no cov
-                    self.display_info(line, end='')
-                    continue
-
-                method, args, kwargs = pickle.loads(bytes.fromhex(procedure.rstrip()))  # noqa: S301
-                if method == 'abort':
-                    process.communicate()
-
-                getattr(self, method)(*args, **kwargs)
-
-        if process.returncode:
-            self.abort(code=process.returncode)
-
-    def read_builder(self, process):
-        import pickle
-
-        lines = []
-        with process:
-            for line in self.platform.stream_process_output(process):
-                indicator, _, procedure = line.partition(':')
-                if indicator != '__HATCH__':  # no cov
-                    lines.append(line)
-                else:
-                    _, args, _ = pickle.loads(bytes.fromhex(procedure))  # noqa: S301
-                    lines.append(args[0])
-
-        output = ''.join(lines)
-        if process.returncode:
-            self.abort(output, code=process.returncode)
-
-        return output
-
     def ensure_environment_plugin_dependencies(self) -> None:
         self.ensure_plugin_dependencies(
             self.project.config.env_requires_complex, wait_message='Syncing environment plugin requirements'
@@ -196,10 +201,9 @@ class Application(Terminal):
         # Default to -1 verbosity
         add_verbosity_flag(command, self.verbosity, adjustment=-1)
 
-        for dependency in dependencies:
-            command.append(str(dependency))
+        command.extend(str(dependency) for dependency in dependencies)
 
-        with self.status_waiting(wait_message):
+        with self.status(wait_message):
             self.platform.check_command(command)
 
     def get_env_directory(self, environment_type):
@@ -209,10 +213,36 @@ class Application(Terminal):
             path = Path(directories[environment_type]).expand()
             if os.path.isabs(path):
                 return path
-            else:
-                return self.project.location / path
-        else:
-            return self.data_dir / 'env' / environment_type
+
+            return self.project.location / path
+
+        return self.data_dir / 'env' / environment_type
+
+    def get_python_manager(self, directory: str | None = None):
+        from hatch.python.core import PythonManager
+
+        configured_dir = directory or self.config.dirs.python
+        if configured_dir == 'shared':
+            return PythonManager(Path.home() / '.pythons')
+
+        if configured_dir == 'isolated':
+            return PythonManager(self.data_dir / 'pythons')
+
+        return PythonManager(Path(configured_dir).expand())
+
+    @cached_property
+    def shell_data(self) -> tuple[str, str]:
+        import shellingham
+
+        try:
+            return shellingham.detect_shell()
+        except shellingham.ShellDetectionFailure:
+            path = self.platform.default_shell
+            return Path(path).stem, path
+
+    @cached_property
+    def env_metadata(self) -> EnvironmentMetadata:
+        return EnvironmentMetadata(self.data_dir / 'env' / '.metadata', self.project.location)
 
     def abort(self, text='', code=1, **kwargs):
         if text:
@@ -238,5 +268,53 @@ class SafeApplication:
         # Divergence from what the backend provides
         self.prompt = app.prompt
         self.confirm = app.confirm
-        self.status_waiting = app.status_waiting
-        self.read_builder = app.read_builder
+        self.status = app.status
+        self.status_if = app.status_if
+
+
+class EnvironmentMetadata:
+    def __init__(self, data_dir: Path, project_path: Path):
+        self.__data_dir = data_dir
+        self.__project_path = project_path
+
+    def dependency_hash(self, environment: EnvironmentInterface) -> str:
+        return self._read(environment).get('dependency_hash', '')
+
+    def update_dependency_hash(self, environment: EnvironmentInterface, dependency_hash: str) -> None:
+        metadata = self._read(environment)
+        metadata['dependency_hash'] = dependency_hash
+        self._write(environment, metadata)
+
+    def reset(self, environment: EnvironmentInterface) -> None:
+        self._metadata_file(environment).unlink(missing_ok=True)
+
+    def _read(self, environment: EnvironmentInterface) -> dict[str, Any]:
+        import json
+
+        metadata_file = self._metadata_file(environment)
+        if not metadata_file.is_file():
+            return {}
+
+        return json.loads(metadata_file.read_text())
+
+    def _write(self, environment: EnvironmentInterface, metadata: dict[str, Any]) -> None:
+        import json
+
+        metadata_file = self._metadata_file(environment)
+        metadata_file.parent.ensure_dir_exists()
+        metadata_file.write_text(json.dumps(metadata))
+
+    def _metadata_file(self, environment: EnvironmentInterface) -> Path:
+        from hatch.env.internal.interface import InternalEnvironment
+
+        if isinstance(environment, InternalEnvironment) and environment.config.get('skip-install'):
+            return self.__data_dir / '.internal' / f'{environment.name}.json'
+
+        return self._storage_dir / environment.config['type'] / f'{environment.name}.json'
+
+    @cached_property
+    def _storage_dir(self) -> Path:
+        from base64 import urlsafe_b64encode
+        from hashlib import sha256
+
+        return self.__data_dir / urlsafe_b64encode(sha256(str(self.__project_path).encode()).digest())[:8].decode()
