@@ -1,12 +1,18 @@
+from __future__ import annotations
+
 import re
 from copy import deepcopy
 from itertools import product
 from os import environ
+from typing import TYPE_CHECKING
 
 from hatch.env.utils import ensure_valid_environment
 from hatch.project.env import apply_overrides
 from hatch.project.utils import format_script_commands, parse_script_command
 
+if TYPE_CHECKING:
+    from packaging.requirements import Requirement
+
 
 class ProjectConfig:
     def __init__(self, root, config, plugin_manager=None):
@@ -16,11 +22,14 @@ class ProjectConfig:
 
         self._matrices = None
         self._env = None
+        self._env_requires_complex = None
+        self._env_requires = None
         self._env_collectors = None
         self._envs = None
         self._matrix_variables = None
         self._publish = None
         self._scripts = None
+        self._fmt = None
         self._cached_env_overrides = {}
 
     @property
@@ -28,23 +37,60 @@ class ProjectConfig:
         if self._env is None:
             config = self.config.get('env', {})
             if not isinstance(config, dict):
-                raise TypeError('Field `tool.hatch.env` must be a table')
+                message = 'Field `tool.hatch.env` must be a table'
+                raise TypeError(message)
 
             self._env = config
 
         return self._env
 
+    @property
+    def env_requires_complex(self) -> list[Requirement]:
+        if self._env_requires_complex is None:
+            from packaging.requirements import InvalidRequirement, Requirement
+
+            requires = self.env.get('requires', [])
+            if not isinstance(requires, list):
+                message = 'Field `tool.hatch.env.requires` must be an array'
+                raise TypeError(message)
+
+            requires_complex = []
+
+            for i, entry in enumerate(requires, 1):
+                if not isinstance(entry, str):
+                    message = f'Requirement #{i} in `tool.hatch.env.requires` must be a string'
+                    raise TypeError(message)
+
+                try:
+                    requires_complex.append(Requirement(entry))
+                except InvalidRequirement as e:
+                    message = f'Requirement #{i} in `tool.hatch.env.requires` is invalid: {e}'
+                    raise ValueError(message) from None
+
+            self._env_requires_complex = requires_complex
+
+        return self._env_requires_complex
+
+    @property
+    def env_requires(self):
+        if self._env_requires is None:
+            self._env_requires = [str(r) for r in self.env_requires_complex]
+
+        return self._env_requires
+
     @property
     def env_collectors(self):
         if self._env_collectors is None:
             collectors = self.env.get('collectors', {})
             if not isinstance(collectors, dict):
-                raise TypeError('Field `tool.hatch.env.collectors` must be a table')
+                message = 'Field `tool.hatch.env.collectors` must be a table'
+                raise TypeError(message)
 
             final_config = {'default': {}}
             for collector, config in collectors.items():
                 if not isinstance(config, dict):
-                    raise TypeError(f'Field `tool.hatch.env.collectors.{collector}` must be a table')
+                    message = f'Field `tool.hatch.env.collectors.{collector}` must be a table'
+                    raise TypeError(message)
 
                 final_config[collector] = config
 
@@ -73,7 +119,8 @@ class ProjectConfig:
         if self._envs is None:
             env_config = self.config.get('envs', {})
             if not isinstance(env_config, dict):
-                raise TypeError('Field `tool.hatch.envs` must be a table')
+                message = 'Field `tool.hatch.envs` must be a table'
+                raise TypeError(message)
 
             config = {}
             environment_collectors = []
@@ -83,7 +130,8 @@ class ProjectConfig:
                 if collector_class is None:
                     from hatchling.plugin.exceptions import UnknownPluginError
 
-                    raise UnknownPluginError(f'Unknown environment collector: {collector}')
+                    message = f'Unknown environment collector: {collector}'
+                    raise UnknownPluginError(message)
 
                 environment_collector = collector_class(self.root, collector_config)
                 environment_collectors.append(environment_collector)
@@ -93,7 +141,8 @@ class ProjectConfig:
 
             for env_name, data in env_config.items():
                 if not isinstance(data, dict):
-                    raise TypeError(f'Field `tool.hatch.envs.{env_name}` must be a table')
+                    message = f'Field `tool.hatch.envs.{env_name}` must be a table'
+                    raise TypeError(message)
 
                 config.setdefault(env_name, {}).update(data)
 
@@ -113,7 +162,7 @@ class ProjectConfig:
             generated_envs = {}
             final_config = {}
             cached_overrides = {}
-            for env_name, initial_config in config.items():
+            for env_name, raw_initial_config in config.items():
                 current_cached_overrides = cached_overrides[env_name] = {
                     'platform': [],
                     'env': [],
@@ -122,32 +171,37 @@ class ProjectConfig:
                 }
 
                 # Only shallow copying is necessary since we just want to modify keys
-                initial_config = initial_config.copy()
+                initial_config = raw_initial_config.copy()
 
                 matrix_name_format = initial_config.pop('matrix-name-format', '{value}')
                 if not isinstance(matrix_name_format, str):
-                    raise TypeError(f'Field `tool.hatch.envs.{env_name}.matrix-name-format` must be a string')
-                elif '{value}' not in matrix_name_format:
-                    raise ValueError(
+                    message = f'Field `tool.hatch.envs.{env_name}.matrix-name-format` must be a string'
+                    raise TypeError(message)
+
+                if '{value}' not in matrix_name_format:
+                    message = (
                         f'Field `tool.hatch.envs.{env_name}.matrix-name-format` must '
                         f'contain at least the `{{value}}` placeholder'
                     )
+                    raise ValueError(message)
 
                 overrides = initial_config.pop('overrides', {})
                 if not isinstance(overrides, dict):
-                    raise TypeError(f'Field `tool.hatch.envs.{env_name}.overrides` must be a table')
+                    message = f'Field `tool.hatch.envs.{env_name}.overrides` must be a table'
+                    raise TypeError(message)
 
                 # Apply any configuration based on the current platform
                 platform_overrides = overrides.get('platform', {})
                 if not isinstance(platform_overrides, dict):
-                    raise TypeError(f'Field `tool.hatch.envs.{env_name}.overrides.platform` must be a table')
+                    message = f'Field `tool.hatch.envs.{env_name}.overrides.platform` must be a table'
+                    raise TypeError(message)
 
                 for platform, options in platform_overrides.items():
                     if not isinstance(options, dict):
-                        raise TypeError(
-                            f'Field `tool.hatch.envs.{env_name}.overrides.platform.{platform}` must be a table'
-                        )
-                    elif platform != current_platform:
+                        message = f'Field `tool.hatch.envs.{env_name}.overrides.platform.{platform}` must be a table'
+                        raise TypeError(message)
+
+                    if platform != current_platform:
                         continue
 
                     apply_overrides(env_name, 'platform', platform, current_platform, options, initial_config)
@@ -156,12 +210,15 @@ class ProjectConfig:
                 # Apply any configuration based on environment variables
                 env_var_overrides = overrides.get('env', {})
                 if not isinstance(env_var_overrides, dict):
-                    raise TypeError(f'Field `tool.hatch.envs.{env_name}.overrides.env` must be a table')
+                    message = f'Field `tool.hatch.envs.{env_name}.overrides.env` must be a table'
+                    raise TypeError(message)
 
                 for env_var, options in env_var_overrides.items():
                     if not isinstance(options, dict):
-                        raise TypeError(f'Field `tool.hatch.envs.{env_name}.overrides.env.{env_var}` must be a table')
-                    elif env_var not in environ:
+                        message = f'Field `tool.hatch.envs.{env_name}.overrides.env.{env_var}` must be a table'
+                        raise TypeError(message)
+
+                    if env_var not in environ:
                         continue
 
                     apply_overrides(env_name, 'env', env_var, environ[env_var], options, initial_config)
@@ -173,58 +230,76 @@ class ProjectConfig:
 
                 matrices = initial_config.pop('matrix')
                 if not isinstance(matrices, list):
-                    raise TypeError(f'Field `tool.hatch.envs.{env_name}.matrix` must be an array')
+                    message = f'Field `tool.hatch.envs.{env_name}.matrix` must be an array'
+                    raise TypeError(message)
 
                 matrix_overrides = overrides.get('matrix', {})
                 if not isinstance(matrix_overrides, dict):
-                    raise TypeError(f'Field `tool.hatch.envs.{env_name}.overrides.matrix` must be a table')
+                    message = f'Field `tool.hatch.envs.{env_name}.overrides.matrix` must be a table'
+                    raise TypeError(message)
 
                 name_overrides = overrides.get('name', {})
                 if not isinstance(name_overrides, dict):
-                    raise TypeError(f'Field `tool.hatch.envs.{env_name}.overrides.name` must be a table')
+                    message = f'Field `tool.hatch.envs.{env_name}.overrides.name` must be a table'
+                    raise TypeError(message)
 
                 matrix_data = all_matrices[env_name] = {'config': deepcopy(initial_config)}
                 all_envs = matrix_data['envs'] = {}
-                for i, matrix in enumerate(matrices, 1):
+                for i, raw_matrix in enumerate(matrices, 1):
+                    matrix = raw_matrix
                     if not isinstance(matrix, dict):
-                        raise TypeError(f'Entry #{i} in field `tool.hatch.envs.{env_name}.matrix` must be a table')
-                    elif not matrix:
-                        raise ValueError(f'Matrix #{i} in field `tool.hatch.envs.{env_name}.matrix` cannot be empty')
+                        message = f'Entry #{i} in field `tool.hatch.envs.{env_name}.matrix` must be a table'
+                        raise TypeError(message)
+
+                    if not matrix:
+                        message = f'Matrix #{i} in field `tool.hatch.envs.{env_name}.matrix` cannot be empty'
+                        raise ValueError(message)
 
                     for j, (variable, values) in enumerate(matrix.items(), 1):
                         if not variable:
-                            raise ValueError(
+                            message = (
                                 f'Variable #{j} in matrix #{i} in field `tool.hatch.envs.{env_name}.matrix` '
                                 f'cannot be an empty string'
                             )
-                        elif not isinstance(values, list):
-                            raise TypeError(
+                            raise ValueError(message)
+
+                        if not isinstance(values, list):
+                            message = (
                                 f'Variable `{variable}` in matrix #{i} in field `tool.hatch.envs.{env_name}.matrix` '
                                 f'must be an array'
                             )
-                        elif not values:
-                            raise ValueError(
+                            raise TypeError(message)
+
+                        if not values:
+                            message = (
                                 f'Variable `{variable}` in matrix #{i} in field `tool.hatch.envs.{env_name}.matrix` '
                                 f'cannot be empty'
                             )
+                            raise ValueError(message)
 
                         existing_values = set()
                         for k, value in enumerate(values, 1):
                             if not isinstance(value, str):
-                                raise TypeError(
+                                message = (
                                     f'Value #{k} of variable `{variable}` in matrix #{i} in field '
                                     f'`tool.hatch.envs.{env_name}.matrix` must be a string'
                                 )
-                            elif not value:
-                                raise ValueError(
+                                raise TypeError(message)
+
+                            if not value:
+                                message = (
                                     f'Value #{k} of variable `{variable}` in matrix #{i} in field '
                                     f'`tool.hatch.envs.{env_name}.matrix` cannot be an empty string'
                                 )
-                            elif value in existing_values:
-                                raise ValueError(
+                                raise ValueError(message)
+
+                            if value in existing_values:
+                                message = (
                                     f'Value #{k} of variable `{variable}` in matrix #{i} in field '
                                     f'`tool.hatch.envs.{env_name}.matrix` is a duplicate'
                                 )
+                                raise ValueError(message)
+
                             existing_values.add(value)
 
                     variables = {}
@@ -234,10 +309,11 @@ class ProjectConfig:
                     for variable in ('py', 'python'):
                         if variable in matrix:
                             if python_selected:
-                                raise ValueError(
+                                message = (
                                     f'Matrix #{i} in field `tool.hatch.envs.{env_name}.matrix` '
                                     f'cannot contain both `py` and `python` variables'
                                 )
+                                raise ValueError(message)
                             python_selected = True
 
                             # Only shallow copying is necessary since we just want to remove a key
@@ -258,10 +334,12 @@ class ProjectConfig:
                         # Apply any configuration based on matrix variables
                         for variable, options in matrix_overrides.items():
                             if not isinstance(options, dict):
-                                raise TypeError(
+                                message = (
                                     f'Field `tool.hatch.envs.{env_name}.overrides.matrix.{variable}` must be a table'
                                 )
-                            elif variable not in variables:
+                                raise TypeError(message)
+
+                            if variable not in variables:
                                 continue
 
                             apply_overrides(
@@ -286,10 +364,10 @@ class ProjectConfig:
                         # Apply any configuration based on the final name, minus the prefix for non-default environments
                         for pattern, options in name_overrides.items():
                             if not isinstance(options, dict):
-                                raise TypeError(
-                                    f'Field `tool.hatch.envs.{env_name}.overrides.name.{pattern}` must be a table'
-                                )
-                            elif not re.search(pattern, new_env_name):
+                                message = f'Field `tool.hatch.envs.{env_name}.overrides.name.{pattern}` must be a table'
+                                raise TypeError(message)
+
+                            if not re.search(pattern, new_env_name):
                                 continue
 
                             apply_overrides(env_name, 'name', pattern, new_env_name, options, new_config)
@@ -331,11 +409,13 @@ class ProjectConfig:
         if self._publish is None:
             config = self.config.get('publish', {})
             if not isinstance(config, dict):
-                raise TypeError('Field `tool.hatch.publish` must be a table')
+                message = 'Field `tool.hatch.publish` must be a table'
+                raise TypeError(message)
 
             for publisher, data in config.items():
                 if not isinstance(data, dict):
-                    raise TypeError(f'Field `tool.hatch.publish.{publisher}` must be a table')
+                    message = f'Field `tool.hatch.publish.{publisher}` must be a table'
+                    raise TypeError(message)
 
             self._publish = config
 
@@ -346,13 +426,15 @@ class ProjectConfig:
         if self._scripts is None:
             script_config = self.config.get('scripts', {})
             if not isinstance(script_config, dict):
-                raise TypeError('Field `tool.hatch.scripts` must be a table')
+                message = 'Field `tool.hatch.scripts` must be a table'
+                raise TypeError(message)
 
             config = {}
 
             for name, data in script_config.items():
                 if ' ' in name:
-                    raise ValueError(f'Script name `{name}` in field `tool.hatch.scripts` must not contain spaces')
+                    message = f'Script name `{name}` in field `tool.hatch.scripts` must not contain spaces'
+                    raise ValueError(message)
 
                 commands = []
 
@@ -361,11 +443,13 @@ class ProjectConfig:
                 elif isinstance(data, list):
                     for i, command in enumerate(data, 1):
                         if not isinstance(command, str):
-                            raise TypeError(f'Command #{i} in field `tool.hatch.scripts.{name}` must be a string')
+                            message = f'Command #{i} in field `tool.hatch.scripts.{name}` must be a string'
+                            raise TypeError(message)
 
                         commands.append(command)
                 else:
-                    raise TypeError(f'Field `tool.hatch.scripts.{name}` must be a string or an array of strings')
+                    message = f'Field `tool.hatch.scripts.{name}` must be a string or an array of strings'
+                    raise TypeError(message)
 
                 config[name] = commands
 
@@ -378,6 +462,18 @@ class ProjectConfig:
 
         return self._scripts
 
+    @property
+    def fmt(self):
+        if self._fmt is None:
+            config = self.config.get('format', {})
+            if not isinstance(config, dict):
+                message = 'Field `tool.hatch.format` must be a table'
+                raise TypeError(message)
+
+            self._fmt = config
+
+        return self._fmt
+
     def finalize_env_overrides(self, option_types):
         # We lazily apply overrides because we need type information potentially defined by
         # environment plugins for their options
@@ -395,9 +491,12 @@ class ProjectConfig:
 def expand_script_commands(script_name, commands, config, seen, active):
     if script_name in seen:
         return seen[script_name]
-    elif script_name in active:
+
+    if script_name in active:
         active.append(script_name)
-        raise ValueError(f'Circular expansion detected for field `tool.hatch.scripts`: {" -> ".join(active)}')
+
+        message = f'Circular expansion detected for field `tool.hatch.scripts`: {" -> ".join(active)}'
+        raise ValueError(message)
 
     active.append(script_name)
 
@@ -426,20 +525,24 @@ def expand_script_commands(script_name, commands, config, seen, active):
 def _populate_default_env_values(env_name, data, config, seen, active):
     if env_name in seen:
         return
-    elif data.get('detached', False):
-        ensure_valid_environment(data)
+
+    if data.pop('detached', False):
         data['template'] = env_name
         data['skip-install'] = True
 
     template_name = data.pop('template', 'default')
     if template_name not in config:
-        raise ValueError(
-            f'Field `tool.hatch.envs.{env_name}.template` refers to an unknown environment `{template_name}`'
-        )
-    elif env_name in active:
+        message = f'Field `tool.hatch.envs.{env_name}.template` refers to an unknown environment `{template_name}`'
+        raise ValueError(message)
+
+    if env_name in active:
         active.append(env_name)
-        raise ValueError(f'Circular inheritance detected for field `tool.hatch.envs.*.template`: {" -> ".join(active)}')
-    elif template_name == env_name:
+
+        message = f'Circular inheritance detected for field `tool.hatch.envs.*.template`: {" -> ".join(active)}'
+        raise ValueError(message)
+
+    if template_name == env_name:
+        ensure_valid_environment(data)
         seen.add(env_name)
         return
 
@@ -451,7 +554,8 @@ def _populate_default_env_values(env_name, data, config, seen, active):
     for key, value in template_config.items():
         if key == 'matrix':
             continue
-        elif key == 'scripts':
+
+        if key == 'scripts':
             scripts = data['scripts'] if 'scripts' in data else data.setdefault('scripts', {})
             for script, commands in value.items():
                 scripts.setdefault(script, commands)
