@@ -524,6 +524,22 @@ class TestCoreMetadataV21:
             """
         )
 
+    def test_license_expression(self, constructor, isolation, helpers):
+        metadata = ProjectMetadata(
+            str(isolation),
+            None,
+            {'project': {'name': 'My.App', 'version': '0.1.0', 'license': 'mit'}},
+        )
+
+        assert constructor(metadata) == helpers.dedent(
+            """
+            Metadata-Version: 2.1
+            Name: My.App
+            Version: 0.1.0
+            License-Expression: MIT
+            """
+        )
+
     def test_keywords_single(self, constructor, isolation, helpers):
         metadata = ProjectMetadata(
             str(isolation), None, {'project': {'name': 'My.App', 'version': '0.1.0', 'keywords': ['foo']}}
@@ -674,7 +690,7 @@ class TestCoreMetadataV21:
             """
         )
 
-    def test_all(self, constructor, isolation, helpers, temp_dir):
+    def test_all(self, constructor, helpers, temp_dir):
         metadata = ProjectMetadata(
             str(temp_dir),
             None,
@@ -930,6 +946,22 @@ class TestCoreMetadataV22:
             """
         )
 
+    def test_license_expression(self, constructor, isolation, helpers):
+        metadata = ProjectMetadata(
+            str(isolation),
+            None,
+            {'project': {'name': 'My.App', 'version': '0.1.0', 'license': 'mit'}},
+        )
+
+        assert constructor(metadata) == helpers.dedent(
+            """
+            Metadata-Version: 2.2
+            Name: My.App
+            Version: 0.1.0
+            License-Expression: MIT
+            """
+        )
+
     def test_keywords_single(self, constructor, isolation, helpers):
         metadata = ProjectMetadata(
             str(isolation), None, {'project': {'name': 'My.App', 'version': '0.1.0', 'keywords': ['foo']}}
@@ -1080,7 +1112,7 @@ class TestCoreMetadataV22:
             """
         )
 
-    def test_all(self, constructor, isolation, helpers, temp_dir):
+    def test_all(self, constructor, helpers, temp_dir):
         metadata = ProjectMetadata(
             str(temp_dir),
             None,
