@@ -16,7 +16,7 @@ def test_default(default_cache_dir, default_data_dir):
         'dirs': {
             'project': [],
             'env': {},
-            'python': 'isolated',
+            'python': 'shared',
             'data': str(default_data_dir),
             'cache': str(default_cache_dir),
         },
@@ -26,7 +26,7 @@ def test_default(default_cache_dir, default_data_dir):
             'name': 'Foo Bar',
             'email': 'foo@bar.baz',
             'licenses': {'default': ['MIT'], 'headers': True},
-            'plugins': {'default': {'ci': False, 'src-layout': False, 'tests': True}},
+            'plugins': {'default': {'ci': False, 'src-layout': True, 'tests': True}},
         },
         'terminal': {
             'styles': {
@@ -348,14 +348,14 @@ class TestDirs:
         default_data_directory = str(default_data_dir)
         assert config.dirs.project == config.dirs.project == []
         assert config.dirs.env == config.dirs.env == {}
-        assert config.dirs.python == config.dirs.python == 'isolated'
+        assert config.dirs.python == config.dirs.python == 'shared'
         assert config.dirs.cache == config.dirs.cache == default_cache_directory
         assert config.dirs.data == config.dirs.data == default_data_directory
         assert config.raw_data == {
             'dirs': {
                 'project': [],
                 'env': {},
-                'python': 'isolated',
+                'python': 'shared',
                 'data': default_data_directory,
                 'cache': default_cache_directory,
             },
@@ -1069,9 +1069,9 @@ class TestTemplate:
     def test_plugins_default(self):
         config = RootConfig({})
 
-        assert config.template.plugins == {'default': {'ci': False, 'src-layout': False, 'tests': True}}
+        assert config.template.plugins == {'default': {'ci': False, 'src-layout': True, 'tests': True}}
         assert config.raw_data == {
-            'template': {'plugins': {'default': {'ci': False, 'src-layout': False, 'tests': True}}}
+            'template': {'plugins': {'default': {'ci': False, 'src-layout': True, 'tests': True}}}
         }
 
     def test_plugins_not_table(self, helpers):
