@@ -4,10 +4,9 @@ import shutil
 import subprocess
 import sys
 import time
-from collections import namedtuple
+from contextlib import suppress
 from functools import lru_cache
-from io import BytesIO
-from typing import Generator
+from typing import Generator, NamedTuple
 
 import pytest
 from click.testing import CliRunner as __CliRunner
@@ -26,7 +25,14 @@ from hatchling.cli import hatchling
 from .helpers.templates.licenses import MIT, Apache_2_0
 
 PLATFORM = Platform()
-DEVPI = namedtuple('DEVPI', ('repo', 'index', 'user', 'auth', 'ca_cert'))
+
+
+class Devpi(NamedTuple):
+    repo: str
+    index_name: str
+    user: str
+    auth: str
+    ca_cert: str
 
 
 class CliRunner(__CliRunner):
@@ -77,29 +83,36 @@ def isolation() -> Generator[Path, None, None]:
             ConfigEnvVars.CACHE: str(cache_dir),
             PublishEnvVars.REPO: 'dev',
             'HATCH_SELF_TESTING': 'true',
+            'PYAPP_COMMAND_NAME': os.urandom(4).hex(),
             'GIT_AUTHOR_NAME': 'Foo Bar',
             'GIT_AUTHOR_EMAIL': 'foo@bar.baz',
             'COLUMNS': '80',
             'LINES': '24',
         }
+        if PLATFORM.windows:
+            default_env_vars['COMSPEC'] = 'cmd.exe'
+        else:
+            default_env_vars['SHELL'] = 'sh'
+
         with d.as_cwd(default_env_vars):
             os.environ.pop(AppEnvVars.ENV_ACTIVE, None)
+            os.environ.pop(AppEnvVars.FORCE_COLOR, None)
             yield d
 
 
 @pytest.fixture(scope='session')
-def isolated_data_dir() -> Generator[Path, None, None]:
-    yield Path(os.environ[ConfigEnvVars.DATA])
+def isolated_data_dir() -> Path:
+    return Path(os.environ[ConfigEnvVars.DATA])
 
 
 @pytest.fixture(scope='session')
-def default_data_dir() -> Generator[Path, None, None]:
-    yield Path(user_data_dir('hatch', appauthor=False))
+def default_data_dir() -> Path:
+    return Path(user_data_dir('hatch', appauthor=False))
 
 
 @pytest.fixture(scope='session')
-def default_cache_dir() -> Generator[Path, None, None]:
-    yield Path(user_cache_dir('hatch', appauthor=False))
+def default_cache_dir() -> Path:
+    return Path(user_cache_dir('hatch', appauthor=False))
 
 
 @pytest.fixture(scope='session')
@@ -157,12 +170,12 @@ def default_virtualenv_installed_requirements(helpers):
         output = PLATFORM.run_command(['pip', 'freeze'], check=True, capture_output=True).stdout.decode('utf-8')
         requirements = helpers.extract_requirements(output.splitlines())
 
-    yield frozenset(requirements)
+    return frozenset(requirements)
 
 
 @pytest.fixture(scope='session')
 def extract_installed_requirements(helpers, default_virtualenv_installed_requirements):
-    yield lambda lines: [
+    return lambda lines: [
         requirement
         for requirement in helpers.extract_requirements(lines)
         if requirement not in default_virtualenv_installed_requirements
@@ -174,9 +187,20 @@ def python_on_path():
     return Path(sys.executable).stem
 
 
+@pytest.fixture(scope='session')
+def compatible_python_distributions():
+    from hatch.python.resolve import get_compatible_distributions
+
+    return tuple(get_compatible_distributions())
+
+
 @pytest.fixture(scope='session')
 def devpi(tmp_path_factory, worker_id):
-    if not shutil.which('docker') or (running_in_ci() and not PLATFORM.linux):
+    import platform
+
+    if not shutil.which('docker') or (
+        running_in_ci() and (not PLATFORM.linux or platform.python_implementation() == 'PyPy')
+    ):
         pytest.skip('Not testing publishing')
 
     # This fixture is affected by https://github.com/pytest-dev/pytest-xdist/issues/271
@@ -210,7 +234,7 @@ def devpi(tmp_path_factory, worker_id):
             server_key = str(server_config_dir / 'server.key')
             server_cert = str(server_config_dir / 'server.pem')
             cert.private_key_pem.write_to_path(path=server_key)
-            with open(server_cert, mode='w') as f:
+            with open(server_cert, mode='w', encoding='utf-8') as f:
                 f.truncate()
             for blob in cert.cert_chain_pems:
                 blob.write_to_path(path=server_cert, append=True)
@@ -222,8 +246,8 @@ def devpi(tmp_path_factory, worker_id):
             data = {'password': os.urandom(16).hex(), 'ca_cert': client_cert}
             devpi_data_file.write_atomic(json.dumps(data), 'w', encoding='utf-8')
 
-    dp = DEVPI('https://localhost:8443/hatch/testing/', 'testing', 'hatch', data['password'], data['ca_cert'])
-    env_vars = {'DEVPI_INDEX_NAME': dp.index, 'DEVPI_USERNAME': dp.user, 'DEVPI_PASSWORD': dp.auth}
+    dp = Devpi('https://localhost:8443/hatch/testing/', 'testing', 'hatch', data['password'], data['ca_cert'])
+    env_vars = {'DEVPI_INDEX_NAME': dp.index_name, 'DEVPI_USERNAME': dp.user, 'DEVPI_PASSWORD': dp.auth}
 
     compose_file = str(devpi_docker_data / 'docker-compose.yaml')
     with FileLock(lock_file):
@@ -231,10 +255,10 @@ def devpi(tmp_path_factory, worker_id):
             with EnvVars(env_vars):
                 subprocess.check_call(['docker', 'compose', '-f', compose_file, 'up', '--build', '-d'])
 
-            for _ in range(30):
+            for _ in range(60):
                 output = subprocess.check_output(['docker', 'logs', 'hatch-devpi']).decode('utf-8')
-                if f'Serving index {dp.user}/{dp.index}' in output:
-                    time.sleep(2)
+                if f'Serving index {dp.user}/{dp.index_name}' in output:
+                    time.sleep(5)
                     break
 
                 time.sleep(1)
@@ -254,7 +278,7 @@ def devpi(tmp_path_factory, worker_id):
                 shutil.rmtree(devpi_ended_sessions)
 
                 with EnvVars(env_vars):
-                    subprocess.run(['docker', 'compose', '-f', compose_file, 'down', '-t', '0'], capture_output=True)
+                    subprocess.run(['docker', 'compose', '-f', compose_file, 'down', '-t', '0'], capture_output=True)  # noqa: PLW1510
 
                 shutil.rmtree(devpi_data)
 
@@ -265,14 +289,11 @@ def mock_backend_process(request, mocker):
         yield False
         return
 
-    line_queue = []
-
     def mock_process_api(api):
         def mock_process(command, **kwargs):
             if not isinstance(command, list) or command[1:4] != ['-u', '-m', 'hatchling']:  # no cov
                 return api(command, **kwargs)
 
-            line_queue.clear()
             original_args = sys.argv
             try:
                 sys.argv = command[3:]
@@ -287,20 +308,91 @@ def mock_backend_process(request, mocker):
                 else:
                     mock.returncode = 0
 
-                mock.stdout = BytesIO(''.join(line_queue).encode('utf-8'))
                 return mock
             finally:
                 sys.argv = original_args
 
         return mock_process
 
-    mocker.patch('subprocess.Popen', side_effect=mock_process_api(subprocess.Popen))
-    mocker.patch('subprocess.run', side_effect=mock_process_api(subprocess.run))
-    mocker.patch('hatchling.bridge.app._send_app_command', side_effect=lambda cmd: line_queue.append(f'{cmd}\n'))
+    mocker.patch('hatch.utils.platform.Platform.run_command', side_effect=mock_process_api(PLATFORM.run_command))
 
     yield True
 
 
+@pytest.fixture
+def mock_backend_process_output(request, mocker):
+    if 'allow_backend_process' in request.keywords:
+        yield False
+        return
+
+    output_queue = []
+
+    def mock_process_api(api):
+        def mock_process(command, **kwargs):
+            if not isinstance(command, list) or command[1:4] != ['-u', '-m', 'hatchling']:  # no cov
+                return api(command, **kwargs)
+
+            output_queue.clear()
+            original_args = sys.argv
+            try:
+                sys.argv = command[3:]
+                mock = mocker.MagicMock()
+
+                try:
+                    # The builder sets process-wide environment variables
+                    with EnvVars():
+                        hatchling()
+                except SystemExit as e:
+                    mock.returncode = e.code
+                else:
+                    mock.returncode = 0
+
+                mock.stdout = ''.join(output_queue).encode('utf-8')
+                return mock
+            finally:
+                sys.argv = original_args
+
+        return mock_process
+
+    mocker.patch('subprocess.run', side_effect=mock_process_api(subprocess.run))
+    mocker.patch('hatchling.bridge.app._display', side_effect=lambda cmd: output_queue.append(f'{cmd}\n'))
+
+    yield True
+
+
+@pytest.fixture
+def mock_plugin_installation(mocker):
+    subprocess_run = subprocess.run
+    mocked_subprocess_run = mocker.MagicMock(returncode=0)
+
+    def _mock(command, **kwargs):
+        if not isinstance(command, list) or command[:5] != [sys.executable, '-u', '-m', 'pip', 'install']:  # no cov
+            return subprocess_run(command, **kwargs)
+
+        mocked_subprocess_run(command, **kwargs)
+        return mocked_subprocess_run
+
+    mocker.patch('subprocess.run', side_effect=_mock)
+
+    return mocked_subprocess_run
+
+
+@pytest.fixture
+def local_backend_process(mock_backend_process, mocker):
+    if mock_backend_process:
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.build_environment')
+
+    return mock_backend_process
+
+
+@pytest.fixture
+def local_backend_process_output(mock_backend_process_output, mocker):
+    if mock_backend_process_output:
+        mocker.patch('hatch.env.virtual.VirtualEnvironment.build_environment')
+
+    return mock_backend_process_output
+
+
 def pytest_runtest_setup(item):
     for marker in item.iter_markers():
         if marker.name == 'requires_internet' and not network_connectivity():  # no cov
@@ -321,6 +413,9 @@ def pytest_runtest_setup(item):
         if marker.name == 'requires_unix' and PLATFORM.windows:
             pytest.skip('Not running on a Linux-based platform')
 
+        if marker.name == 'requires_git' and not shutil.which('git'):
+            pytest.skip('Git not present in the environment')
+
 
 def pytest_configure(config):
     config.addinivalue_line('markers', 'requires_windows: Tests intended for Windows operating systems')
@@ -328,6 +423,7 @@ def pytest_configure(config):
     config.addinivalue_line('markers', 'requires_linux: Tests intended for Linux operating systems')
     config.addinivalue_line('markers', 'requires_unix: Tests intended for Linux-based operating systems')
     config.addinivalue_line('markers', 'requires_internet: Tests that require access to the internet')
+    config.addinivalue_line('markers', 'requires_git: Tests that require the git command available in the environment')
 
     config.addinivalue_line('markers', 'allow_backend_process: Force the use of backend communication')
 
@@ -335,18 +431,18 @@ def pytest_configure(config):
     config.getini('norecursedirs').remove('venv')  # /tests/venv
 
 
-@lru_cache()
+@lru_cache
 def network_connectivity():  # no cov
     if running_in_ci():
         return True
 
     import socket
 
-    try:
+    with suppress(Exception):
         # Test availability of DNS first
         host = socket.gethostbyname('www.google.com')
         # Test connection
         socket.create_connection((host, 80), 2)
         return True
-    except Exception:
-        return False
+
+    return False
